@@ -26,14 +26,14 @@ class PubTatorLoader(CollLoader):
 
     _section_labels = {'t': 'Title', 'a': 'Abstract'}
 
-    def __init__(self, info=('type', 'cui')):
+    def __init__(self, meta=('type', 'cui')):
         (self.type,
-         self.cui) = info
+         self.cui) = meta
 
-    def collection(self, source, id_):
+    def collection(self, source, id):
         entity_counter = it.count(1)
         docs = self._iter_documents(source, entity_counter)
-        return Collection.from_iterable(docs, id_, basename(source))
+        return Collection.from_iterable(docs, id, basename(source))
 
     def iter_documents(self, source):
         return self._iter_documents(source)
@@ -61,7 +61,11 @@ class PubTatorLoader(CollLoader):
         docid, sections, anno = self._parse(lines, entity_counter)
         doc = Document(docid)
         for label, text in sections:
-            doc.add_section(label, text)
+            # Provide entities of the entire document in an iterator.
+            # Like this, they will be used for preventing sentence boundaries
+            # within annotations, but won't be added to the sentences yet
+            # (which would cause out-of-range exceptions).
+            doc.add_section(label, text, entities=iter(anno), entity_offset=0)
         doc.add_entities(anno)
         return doc
 
@@ -106,10 +110,10 @@ class PubTatorLoader(CollLoader):
         return label, text
 
     def _entity(self, ids, start, end, text, type_, cui=None):
-        info = {self.type: type_}
+        meta = {self.type: type_}
         if cui is not None:
-            info[self.cui] = cui
-        return Entity(next(ids), text, int(start), int(end), info)
+            meta[self.cui] = cui
+        return Entity(next(ids), text, [(int(start), int(end))], meta)
 
 
 class PubTatorFBKLoader(PubTatorLoader):
@@ -117,8 +121,8 @@ class PubTatorFBKLoader(PubTatorLoader):
     Load FBK-flavored PubTator documents.
     """
 
-    def __init__(self, info='type'):
-        super().__init__([info, None])
+    def __init__(self, meta='type'):
+        super().__init__([meta, None])
         del self.cui
 
     def _entity(self, _, id_, type_, start, end, text):
@@ -126,8 +130,8 @@ class PubTatorFBKLoader(PubTatorLoader):
             id_ = int(id_.lstrip('T'))
         except ValueError:
             pass
-        info = {self.type: type_}
-        return Entity(id_, text, int(start), int(end), info)
+        meta = {self.type: type_}
+        return Entity(id_, text, [(int(start), int(end))], meta)
 
 
 class PubTatorFormatter(StreamFormatter):
@@ -137,9 +141,9 @@ class PubTatorFormatter(StreamFormatter):
 
     ext = 'txt'
 
-    def __init__(self, info=('type', 'cui')):
+    def __init__(self, meta=('type', 'cui')):
         (self.type,
-         self.cui) = info
+         self.cui) = meta
 
     def write(self, content, stream):
         tsv = csv.writer(stream, **tsv_format)
@@ -183,13 +187,27 @@ class PubTatorFormatter(StreamFormatter):
         return '|'.join((id_, t, text))
 
     def _annotations(self, docid, sec, offset):
-        for entity in sec.iter_entities():
+        for entity in sec.iter_entities(split_discontinuous=True):
             start, end = entity.start+offset, entity.end+offset
             yield self._select_anno_fields(docid, start, end, entity)
 
     def _select_anno_fields(self, docid, start, end, entity):
-        return (docid, start, end, entity.text_wn,
-                entity.info[self.type], entity.info[self.cui])
+        return (docid, start, end, entity.text_wn, *self._entity_meta(entity))
+
+    def _entity_meta(self, entity):
+        for key in (self.type, self.cui):
+            try:
+                yield entity.metadata[key]
+            except KeyError as e:
+                if e.args == (self.cui,):
+                    return  # CUI is optional; ignore silently.
+                if e.args == (key,):
+                    raise ValueError(
+                        'Need entity attribute: '
+                        '{!r} not found in Entity.metadata. '
+                        'Please check the `meta` option.'
+                        .format(key))
+                raise
 
 
 class PubTatorFBKFormatter(PubTatorFormatter):
@@ -197,9 +215,8 @@ class PubTatorFBKFormatter(PubTatorFormatter):
     FBK flavor of the PubTator format.
     """
 
-    def __init__(self, info='type'):
-        super().__init__([info, None])
-        del self.cui
+    def __init__(self, meta='type'):
+        super().__init__([meta, None])
 
     def _select_anno_fields(self, docid, start, end, entity):
         try:
@@ -208,4 +225,8 @@ class PubTatorFBKFormatter(PubTatorFormatter):
             id_ = entity.id
         else:
             id_ = 'T{}'.format(id_)
-        return (docid, id_, entity.info[self.type], start, end, entity.text)
+        return (docid, id_, self._entity_type(entity), start, end, entity.text)
+
+    def _entity_type(self, entity):
+        type_, *_ = self._entity_meta(entity)
+        return type_
