@@ -24,7 +24,16 @@ def ngrams(iterable, n, pad_left=False, pad_right=False,
             yield tuple(window)
     for _ in range((n-1)*pad_right):
         window.append(right_pad_symbol)
-        yield tuple(window)
+        if len(window) == n:
+            yield tuple(window)
+
+
+def pids(prefix, start=1):
+    """
+    Iterate over successive IDs with an arbitrary prefix.
+    """
+    for i in it.count(start):
+        yield '{}{}'.format(prefix, i)
 
 
 def peek(iterable):
